from typing import Optional

from sklearn import ensemble, linear_model

try:
    from xgboost import XGBRegressor
    from xgboost import XGBClassifier
    from xgboost import XGBRFRegressor, XGBRFClassifier

    from lightgbm import LGBMClassifier, LGBMRegressor
except ImportError:
    raise ImportError("the menu_hurdle_estimator requires xboost and lightgbm. Please install before running") 

from . import hurdle

class MenuHurdleEstimator(hurdle.HurdleEstimator):
    def __init__(self,
                 clf_name: str = 'logistic',
                 reg_name: str = 'linear',
                 clf_params: Optional[dict] = None,
                 reg_params: Optional[dict] = None,
                 n_jobs: int = 4):

        self._n_jobs = n_jobs

        threshold_estimator = self._resolve_estimator(clf_name)
        threshold_estimator.set_params(**clf_params)

        regression_estimator = self._resolve_estimator(reg_name)
        regression_estimator.set_params(**reg_params)

        super().__init__(threshold_estimator, regression_estimator)

    def _resolve_estimator(self, func_name: str):
        """ Lookup table for supported estimators.
        This is necessary because sklearn estimator default arguments
        must pass equality test, and instantiated sub-estimators are not equal. """

        funcs = {'linear':         linear_model.LinearRegression(),
                 'logistic':       linear_model.LogisticRegression(solver = 'liblinear'),
                 'LGBMRegressor':  LGBMRegressor(n_estimators = 100),
                 'LGBMClassifier': LGBMClassifier(n_estimators = 100),
                 'RFRegressor':    XGBRFRegressor(n_estimators = 300, n_jobs = self._n_jobs),
                 'RFClassifier':   XGBRFClassifier(n_estimators = 300, n_jobs = self._n_jobs),
                 'GBMRegressor':   ensemble.GradientBoostingRegressor(n_estimators = 200),
                 'GBMClassifier':  ensemble.GradientBoostingClassifier(n_estimators = 200),
                 'XGBRegressor':   XGBRegressor(n_estimators = 200, tree_method = 'hist', n_jobs = self._n_jobs),
                 'XGBClassifier':  XGBClassifier(n_estimators = 200, tree_method = 'hist', n_jobs = self._n_jobs, use_label_encoder = False),
                 'HGBRegressor':   ensemble.HistGradientBoostingRegressor(max_iter = 200),
                 'HGBClassifier':  ensemble.HistGradientBoostingClassifier(max_iter = 200),
                }

        return funcs[func_name]




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hurdle']

package_data = \
{'': ['*']}

install_requires = \
['scikit-learn>=1.0.2,<2.0.0']

setup_kwargs = {
    'name': 'hurdle',
    'version': '1.0.0',
    'description': 'Scikit hurdle estimator',
    'long_description': '\n# Hurdle\n\n![tests status](https://github.com/prio-data/cc_backend_lib/actions/workflows/test.yml/badge.svg)\n\nThis package contains an implementation of Hurdle Regression, based in part on\n[Geoff Ruddocks implementation](https://geoffruddock.com/building-a-hurdle-regression-estimator-in-scikit-learn/)\nand Håvard Hegres 2022 adaption of his implementation.\n\n## Usage\n\n```\nfrom hurdle import HurdleEstimator\nfrom sklearn import linear_model\n\nest = HurdleEstimator(linear_model.LogisticRegression(), linear_model.LinearModel())\n\nest.fit(...)\n```\n',
    'author': 'peder2911',
    'author_email': 'pglandsverk@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://www.github.com/prio-data/hurdle',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)




import numpy as np
from sklearn import base
from sklearn.utils import validation
from sklearn.exceptions import NotFittedError

class HurdleEstimator(base.BaseEstimator):

    def __init__(self,
            threshold_estimator: base.BaseEstimator,
            regression_estimator: base.BaseEstimator):

        self.threshold_estimator = threshold_estimator
        self.regression_estimator = regression_estimator

    def fit(self,
            X: np.ndarray,
            y: np.ndarray):

        X, y = validation.check_X_y(X, y, dtype=None,
                         accept_sparse=False,
                         accept_large_sparse=False,
                         force_all_finite='allow-nan')

        if X.shape[1] < 2:
            raise ValueError('Cannot fit model when n_features = 1')

        self.threshold_estimator.fit(X, y > 0)
        self.regression_estimator.fit(X[y > 0], y[y > 0])

        return self

    def _predict(self, X: np.ndarray, binary: bool = False):
        X = validation.check_array(X, accept_sparse=False, accept_large_sparse=False)
        validation.check_is_fitted(self)

        threshold_predict_method = "predict_proba" if not binary else "predict"
        threshold_predict = getattr(self.threshold_estimator, threshold_predict_method)
        threshold = threshold_predict(X)[:, 1]
        regression = self.regression_estimator.predict(X)
        return threshold * regression

    def predict(self, X: np.ndarray):
        """ Predict combined response using probabilistic classification outcome """
        return self._predict(X)

    def predict_bck(self, X: np.ndarray):
        """ Predict combined response using binary classification outcome """
        return self._predict(X, binary = True)

    def __sklearn_is_fitted__(self):
        return (
            self._is_fitted(self.regression_estimator) &
            self._is_fitted(self.threshold_estimator))

    @staticmethod
    def _is_fitted(estimator: base.BaseEstimator) -> bool:
        try:
            validation.check_is_fitted(estimator)
            return True
        except NotFittedError:
            return False
